@@ -29,8 +29,14 @@ USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "common.h"
 
 
-#if defined(BULLDOZER) || defined(PILEDRIVER) || defined(STEAMROLLER)
+#if defined(PILEDRIVER) || defined(STEAMROLLER)
+#include "caxpy_microk_steamroller-2.c"
+#elif defined(BULLDOZER)
 #include "caxpy_microk_bulldozer-2.c"
+#elif defined(HASWELL)
+#include "caxpy_microk_haswell-2.c"
+#elif defined(SANDYBRIDGE)
+#include "caxpy_microk_sandy-2.c"
 #endif
 
 
@@ -78,15 +84,16 @@ int CNAME(BLASLONG n, BLASLONG dummy0, BLASLONG dummy1, FLOAT da_r, FLOAT da_i,
 	if ( (inc_x == 1) && (inc_y == 1) )
 	{
 
-		int n1 = n & -8;
+		BLASLONG n1 = n & -32;
 
 		if ( n1 )
 		{
 			da[0] = da_r;
 			da[1] = da_i;
-			caxpy_kernel_8(n1, x, y , &da );
+			caxpy_kernel_8(n1, x, y , da );
 			ix = 2 * n1;
 		}
+
 		i = n1;
 		while(i < n)
 		{
