@@ -31,89 +31,87 @@ static void caxpy_kernel_8( BLASLONG n, FLOAT *x, FLOAT *y , FLOAT *alpha) __att
 static void caxpy_kernel_8( BLASLONG n, FLOAT *x, FLOAT *y, FLOAT *alpha)
 {
 
+#if !defined(CONJ)
+	FLOAT mvec[4] = { -1.0, 1.0, -1.0, 1.0 };
+#else
+	FLOAT mvec[4] = { 1.0, -1.0, 1.0, -1.0 };
+#endif
 
 	BLASLONG register i = 0;
 
+  if ( n < 640 )
+  {
+
 	__asm__  __volatile__
 	(
+	"vzeroupper					    \n\t"
 	"vbroadcastss		(%4), %%xmm0		    \n\t"  // real part of alpha
 	"vbroadcastss	       4(%4), %%xmm1		    \n\t"  // imag part of alpha
+#if !defined(CONJ)
+	"vmulps		(%5), %%xmm1 , %%xmm1		    \n\t"
+#else
+	"vmulps		(%5), %%xmm0 , %%xmm0		    \n\t"
+#endif
 
 	".align 16				            \n\t"
 	"1:				            \n\t"
 
-	"prefetcht0  768(%2,%0,4)                           \n\t"
 	"vmovups        (%2,%0,4), %%xmm5                   \n\t" // 2 complex values from x
+	".align 2					    \n\t"
 	"vmovups      16(%2,%0,4), %%xmm7                   \n\t" // 2 complex values from x
 	"vmovups      32(%2,%0,4), %%xmm9                   \n\t" // 2 complex values from x
 	"vmovups      48(%2,%0,4), %%xmm11                  \n\t" // 2 complex values from x
-	"prefetcht0  768(%3,%0,4)                           \n\t"
 
-#if !defined(CONJ)
-	"vfmaddps    (%3,%0,4), %%xmm0 , %%xmm5, %%xmm12    \n\t"
+	"vmovups      64(%2,%0,4), %%xmm12                  \n\t" // 2 complex values from x
+	"vmovups      80(%2,%0,4), %%xmm13                  \n\t" // 2 complex values from x
+	"vmovups      96(%2,%0,4), %%xmm14                  \n\t" // 2 complex values from x
+	"vmovups     112(%2,%0,4), %%xmm15                  \n\t" // 2 complex values from x
+
 	"vpermilps	$0xb1 , %%xmm5 , %%xmm4 	    \n\t"  // exchange real and imag part
-	"vmulps         %%xmm1, %%xmm4 , %%xmm4             \n\t"
-
-	"vfmaddps  16(%3,%0,4), %%xmm0 , %%xmm7, %%xmm13    \n\t"
 	"vpermilps	$0xb1 , %%xmm7 , %%xmm6 	    \n\t"  // exchange real and imag part
-	"vmulps         %%xmm1, %%xmm6 , %%xmm6             \n\t"
-
-	"vfmaddps  32(%3,%0,4), %%xmm0 , %%xmm9, %%xmm14    \n\t"
 	"vpermilps	$0xb1 , %%xmm9 , %%xmm8 	    \n\t"  // exchange real and imag part
-	"vmulps         %%xmm1, %%xmm8 , %%xmm8             \n\t"
-
-	"vfmaddps  48(%3,%0,4), %%xmm0 , %%xmm11,%%xmm15    \n\t"
 	"vpermilps	$0xb1 , %%xmm11, %%xmm10 	    \n\t"  // exchange real and imag part
-	"vmulps         %%xmm1, %%xmm10, %%xmm10            \n\t"
 
-        "vaddsubps      %%xmm4, %%xmm12, %%xmm12            \n\t"
-        "vaddsubps      %%xmm6, %%xmm13, %%xmm13            \n\t"
-        "vaddsubps      %%xmm8, %%xmm14, %%xmm14            \n\t"
-        "vaddsubps      %%xmm10,%%xmm15, %%xmm15            \n\t"
+	"vfmaddps    (%3,%0,4), %%xmm0 , %%xmm5, %%xmm5     \n\t"
+	".align 2					    \n\t"
+	"vfmaddps  16(%3,%0,4), %%xmm0 , %%xmm7, %%xmm7     \n\t"
+	"vfmaddps  32(%3,%0,4), %%xmm0 , %%xmm9, %%xmm9     \n\t"
+	"vfmaddps  48(%3,%0,4), %%xmm0 , %%xmm11,%%xmm11    \n\t"
 
-#else
+	"vfmaddps	%%xmm5 , %%xmm1 , %%xmm4 , %%xmm5   \n\t"
+	"vfmaddps	%%xmm7 , %%xmm1 , %%xmm6 , %%xmm7   \n\t"
+	"vfmaddps	%%xmm9 , %%xmm1 , %%xmm8 , %%xmm9   \n\t"
+	"vfmaddps	%%xmm11, %%xmm1 , %%xmm10, %%xmm11  \n\t"
 
-	"vmulps		%%xmm0, %%xmm5, %%xmm4		    \n\t" // a_r*x_r, a_r*x_i
-	"vmulps		%%xmm1, %%xmm5, %%xmm5 		    \n\t" // a_i*x_r, a_i*x_i
-	"vmulps		%%xmm0, %%xmm7, %%xmm6		    \n\t" // a_r*x_r, a_r*x_i
-	"vmulps		%%xmm1, %%xmm7, %%xmm7 		    \n\t" // a_i*x_r, a_i*x_i
-	"vmulps		%%xmm0, %%xmm9, %%xmm8		    \n\t" // a_r*x_r, a_r*x_i
-	"vmulps		%%xmm1, %%xmm9, %%xmm9 		    \n\t" // a_i*x_r, a_i*x_i
-	"vmulps		%%xmm0, %%xmm11, %%xmm10	    \n\t" // a_r*x_r, a_r*x_i
-	"vmulps		%%xmm1, %%xmm11, %%xmm11	    \n\t" // a_i*x_r, a_i*x_i
+	"vpermilps	$0xb1 , %%xmm12, %%xmm4 	    \n\t"  // exchange real and imag part
+	"vpermilps	$0xb1 , %%xmm13, %%xmm6 	    \n\t"  // exchange real and imag part
+	"vpermilps	$0xb1 , %%xmm14, %%xmm8 	    \n\t"  // exchange real and imag part
+	"vpermilps	$0xb1 , %%xmm15, %%xmm10 	    \n\t"  // exchange real and imag part
 
-	"vpermilps	$0xb1 , %%xmm4 , %%xmm4 	    \n\t"  // exchange real and imag part
-	"vaddsubps	%%xmm4 ,%%xmm5 , %%xmm4 	    \n\t"
-	"vpermilps	$0xb1 , %%xmm4 , %%xmm4 	    \n\t"  // exchange real and imag part
+	"vfmaddps  64(%3,%0,4), %%xmm0 , %%xmm12, %%xmm12   \n\t"
+	"vfmaddps  80(%3,%0,4), %%xmm0 , %%xmm13, %%xmm13   \n\t"
+	"vfmaddps  96(%3,%0,4), %%xmm0 , %%xmm14, %%xmm14   \n\t"
+	"vfmaddps 112(%3,%0,4), %%xmm0 , %%xmm15, %%xmm15   \n\t"
 
-	"vpermilps	$0xb1 , %%xmm6 , %%xmm6 	    \n\t"  // exchange real and imag part
-	"vaddsubps	%%xmm6 ,%%xmm7 , %%xmm6 	    \n\t"
-	"vpermilps	$0xb1 , %%xmm6 , %%xmm6 	    \n\t"  // exchange real and imag part
+	"vfmaddps	%%xmm12, %%xmm1 , %%xmm4 , %%xmm12  \n\t"
+	"vfmaddps	%%xmm13, %%xmm1 , %%xmm6 , %%xmm13  \n\t"
+	"vfmaddps	%%xmm14, %%xmm1 , %%xmm8 , %%xmm14  \n\t"
+	"vfmaddps	%%xmm15, %%xmm1 , %%xmm10, %%xmm15  \n\t"
 
-	"vpermilps	$0xb1 , %%xmm8 , %%xmm8 	    \n\t"  // exchange real and imag part
-	"vaddsubps	%%xmm8 ,%%xmm9 , %%xmm8 	    \n\t"
-	"vpermilps	$0xb1 , %%xmm8 , %%xmm8 	    \n\t"  // exchange real and imag part
+	"vmovups	%%xmm5 ,   (%3,%0,4)		    \n\t"
+	".align 2					    \n\t"
+	"vmovups	%%xmm7 , 16(%3,%0,4)		    \n\t"
+	"vmovups	%%xmm9 , 32(%3,%0,4)		    \n\t"
+	"vmovups	%%xmm11, 48(%3,%0,4)		    \n\t"
+	"vmovups	%%xmm12, 64(%3,%0,4)		    \n\t"
+	"vmovups	%%xmm13, 80(%3,%0,4)		    \n\t"
+	"vmovups	%%xmm14, 96(%3,%0,4)		    \n\t"
+	"vmovups	%%xmm15,112(%3,%0,4)		    \n\t"
 
-	"vpermilps	$0xb1 , %%xmm10, %%xmm10	    \n\t"  // exchange real and imag part
-	"vaddsubps	%%xmm10,%%xmm11, %%xmm10	    \n\t"
-	"vpermilps	$0xb1 , %%xmm10, %%xmm10	    \n\t"  // exchange real and imag part
-
-	"vaddps	     (%3,%0,4) ,%%xmm4 , %%xmm12	    \n\t"
-	"vaddps	   16(%3,%0,4) ,%%xmm6 , %%xmm13	    \n\t"
-	"vaddps	   32(%3,%0,4) ,%%xmm8 , %%xmm14	    \n\t"
-	"vaddps	   48(%3,%0,4) ,%%xmm10, %%xmm15	    \n\t"
-
-
-#endif
-
-	"vmovups	%%xmm12,   (%3,%0,4)		    \n\t"
-	"vmovups	%%xmm13, 16(%3,%0,4)		    \n\t"
-	"vmovups	%%xmm14, 32(%3,%0,4)		    \n\t"
-	"vmovups	%%xmm15, 48(%3,%0,4)		    \n\t"
-
-	"addq		$16, %0	  	 	             \n\t"
-	"subq	        $8 , %1			             \n\t"		
+	"addq		$32, %0	  	 	             \n\t"
+	"subq	        $16, %1			             \n\t"		
 	"jnz		1b		             \n\t"
+	"vzeroupper					    \n\t"
 
 	:
         : 
@@ -121,7 +119,8 @@ static void caxpy_kernel_8( BLASLONG n, FLOAT *x, FLOAT *y, FLOAT *alpha)
 	  "r" (n),  	// 1
           "r" (x),      // 2
           "r" (y),      // 3
-          "r" (alpha)   // 4
+          "r" (alpha),  // 4
+          "r" (mvec)    // 5
 	: "cc", 
 	  "%xmm0", "%xmm1",
 	  "%xmm4", "%xmm5", "%xmm6", "%xmm7", 
@@ -129,7 +128,73 @@ static void caxpy_kernel_8( BLASLONG n, FLOAT *x, FLOAT *y, FLOAT *alpha)
 	  "%xmm12", "%xmm13", "%xmm14", "%xmm15",
 	  "memory"
 	);
+	return;
+  }
+
+	__asm__  __volatile__
+	(
+	"vzeroupper					    \n\t"
+	"vbroadcastss		(%4), %%xmm0		    \n\t"  // real part of alpha
+	"vbroadcastss	       4(%4), %%xmm1		    \n\t"  // imag part of alpha
+#if !defined(CONJ)
+	"vmulps		(%5), %%xmm1 , %%xmm1		    \n\t"
+#else
+	"vmulps		(%5), %%xmm0 , %%xmm0		    \n\t"
+#endif
+
+	".align 16				            \n\t"
+	"1:				            \n\t"
+
+	"prefetcht0	512(%2,%0,4)			    \n\t"
+	"vmovups        (%2,%0,4), %%xmm5                   \n\t" // 2 complex values from x
+	".align 2					    \n\t"
+	"vmovups      16(%2,%0,4), %%xmm7                   \n\t" // 2 complex values from x
+	"vmovups      32(%2,%0,4), %%xmm9                   \n\t" // 2 complex values from x
+	"vmovups      48(%2,%0,4), %%xmm11                  \n\t" // 2 complex values from x
+
+	"vpermilps	$0xb1 , %%xmm5 , %%xmm4 	    \n\t"  // exchange real and imag part
+	"vpermilps	$0xb1 , %%xmm7 , %%xmm6 	    \n\t"  // exchange real and imag part
+	"vpermilps	$0xb1 , %%xmm9 , %%xmm8 	    \n\t"  // exchange real and imag part
+	"vpermilps	$0xb1 , %%xmm11, %%xmm10 	    \n\t"  // exchange real and imag part
+
+	"prefetcht0	512(%3,%0,4)			    \n\t"
+	"vfmaddps    (%3,%0,4), %%xmm0 , %%xmm5, %%xmm5     \n\t"
+	".align 2					    \n\t"
+	"vfmaddps  16(%3,%0,4), %%xmm0 , %%xmm7, %%xmm7     \n\t"
+	"vfmaddps  32(%3,%0,4), %%xmm0 , %%xmm9, %%xmm9     \n\t"
+	"vfmaddps  48(%3,%0,4), %%xmm0 , %%xmm11,%%xmm11    \n\t"
+
+	"vfmaddps	%%xmm5 , %%xmm1 , %%xmm4 , %%xmm5   \n\t"
+	"vfmaddps	%%xmm7 , %%xmm1 , %%xmm6 , %%xmm7   \n\t"
+	"vfmaddps	%%xmm9 , %%xmm1 , %%xmm8 , %%xmm9   \n\t"
+	"vfmaddps	%%xmm11, %%xmm1 , %%xmm10, %%xmm11  \n\t"
+
+	"vmovups	%%xmm5 ,   (%3,%0,4)		    \n\t"
+	".align 2					    \n\t"
+	"vmovups	%%xmm7 , 16(%3,%0,4)		    \n\t"
+	"vmovups	%%xmm9 , 32(%3,%0,4)		    \n\t"
+	"vmovups	%%xmm11, 48(%3,%0,4)		    \n\t"
+
+	"addq		$16, %0	  	 	             \n\t"
+	"subq	        $8, %1			             \n\t"		
+	"jnz		1b		             \n\t"
+	"vzeroupper					    \n\t"
+
+	:
+        : 
+          "r" (i),	// 0	
+	  "r" (n),  	// 1
+          "r" (x),      // 2
+          "r" (y),      // 3
+          "r" (alpha),  // 4
+          "r" (mvec)    // 5
+	: "cc", 
+	  "%xmm0", "%xmm1",
+	  "%xmm4", "%xmm5", "%xmm6", "%xmm7", 
+	  "%xmm8", "%xmm9", "%xmm10", "%xmm11", 
+	  "memory"
+	);
+
 
 } 
 
-
